# ---------------------------------------------------------------------------

# Import system modules
import sys, string, math, time, os
import win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

#9.2
#import arcgisscripting
#gp = arcgisscripting.create()

# Check out any necessary licenses
gp.CheckOutExtension("spatial")


# Script arguments...
theme = sys.argv[1]
#Zone_field = sys.argv[2]
Zone_field = "PARCEL_ID"
rstrs = sys.argv[2]
oroWorkSpace = sys.argv[3]
thresh = string.atof(sys.argv[4])
#centroid_opt = sys.argv[6]
centroid_opt = 'false'
oFile = sys.argv[5]

Ignore_NoData_in_calculations = "true"

#check for field type - ESRI bug!!
f = gp.ListFields(theme)
ff = f.Next()
while ff:
    if ff.Name == Zone_field:
        if ff.Type == "String":
            link_id = Zone_field
        else:
            link_id = "VALUE_"
    ff = f.Next()

try:
    wksp = os.path.dirname(oFile)

    gp.CreatePersonalGDB(wksp, "temp_wts.mdb")
    gp.Workspace = (wksp + os.sep + "temp_wts.mdb")

    
    desc = gp.Describe(theme)
    type = desc.ShapeType

    if type != "point":
        if (centroid_opt != 'true'):
            gp.CopyFeatures(theme, "parcels")
            stats = "Shape_Area sum"
            for i in range(9):
                stats = stats + "; SW_WDID" + str(i+1) + " first"
                stats = stats + "; SW_COV" + str(i+1) + " first"
            gp.Statistics("parcels", "wtsTable",stats,Zone_field)
            gp.AddField("wtsTable","TOTAL", "FLOAT")
            gp.CalculateField("wtsTable","TOTAL",0)
        else:
            gp.CopyFeatures(theme, "parcels")
            gp.addmessage("Computing polygon centroids")
            gp.CreateFeatureClass(wksp + os.sep + "temp_wts.mdb","wtsTable","point","#","#","#",gp.Describe(theme).SpatialReference)    
            gp.AddField ("wtsTable", Zone_field, "text", "12")
            gp.AddField("wtsTable","TOTAL", "FLOAT")
            gp.CalculateField("wtsTable","TOTAL",0)
            cenCur = gp.InsertCursor("wtsTable")

            p = gp.CreateObject("point")
            pRows = gp.SearchCursor("parcels")
            pRow = pRows.next()
            while pRow:
                pp = string.split(pRow.Shape.Centroid," ")
                p.x = string.atof(pp[0])
                p.y = string.atof(pp[1])
                cenRow = cenCur.NewRow()
                cenRow.Shape = p
                cenRow.SetValue(Zone_field,pRow.GetValue(Zone_field))
                cenCur.InsertRow(cenRow)
                pRow = pRows.next()

            del pRow,pRows,cenCur,cenRow,p

    else:
        gp.CopyFeatures(theme, "wtsTable")
        
    gp.MakeTableView("wtsTable", "wtsTable_vw")

    rasters = rstrs.split(";")
    
    gp.AddMessage("Processing Climate Station grids")        
    
    for rast in rasters:
        name = os.path.basename(rast)
        gp.AddMessage("Processing GRID " + name)

        if (type == "point" or centroid_opt == 'true'):
            gp.AddMessage("Processing point values")
            gp.ExtractValuesToPoints("wtsTable", rast, name, "INTERPOLATE", "VALUE_ONLY")        
            # Process: Add Field...
            gp.AddField(name,"R_"+name, "FLOAT")
            gp.CalculateField(name,"R_"+name,"[RASTERVALU]")

            gp.DeleteField(name,"RASTERVALU")
            gp.DeleteField(name,"OBJECTID")

            gp.CopyFeatures(name,"wtsTable")
            time.sleep(5);
            gp.Delete(name)
            if oroWorkSpace != "#":
                gp.AddMessage("Processing Orographic Correction")
                gp.ExtractValuesToPoints("wtsTable", oroWorkSpace + os.sep + name, name, "INTERPOLATE", "VALUE_ONLY")        
                # Process: Add Field...
                gp.AddField(name,"O_"+name, "FLOAT")
                gp.CalculateField(name,"O_"+name,"[RASTERVALU]")

                gp.DeleteField(name,"RASTERVALU")
                gp.DeleteField(name,"OBJECTID")

                gp.CopyFeatures(name,"wtsTable")
                time.sleep(5);
                gp.Delete(name)

        else:
            gp.ZonalStatisticsAsTable("parcels", Zone_field, rast, name, Ignore_NoData_in_calculations)            
            gp.AddField("wtsTable","R_"+name, "FLOAT")
            gp.AddJoin("wtsTable_vw",Zone_field,name,link_id)
            #gp.SelectLayerByAttribute("wtsTable_vw", "NEW_SELECTION", name + ".MEAN IS NOT NULL")
            gp.CalculateField("wtsTable_vw","R_"+name,"[" + name + ".MEAN]")
            gp.RemoveJoin("wtsTable_vw", name)
            time.sleep(5);
            gp.Delete(name)
            if oroWorkSpace != "#":
                gp.AddMessage("Processing Orographic Correction")
                gp.ZonalStatisticsAsTable("parcels", Zone_field, oroWorkSpace + os.sep + name, name, Ignore_NoData_in_calculations)            
                gp.AddField("wtsTable","O_"+name, "FLOAT")
                gp.AddJoin("wtsTable_vw",Zone_field,name,link_id)
                #gp.SelectLayerByAttribute("wtsTable_vw", "NEW_SELECTION", name + ".MEAN IS NOT NULL")
                gp.CalculateField("wtsTable_vw","O_"+name,"[" + name + ".MEAN]")
                gp.RemoveJoin("wtsTable_vw", name)
                time.sleep(5);
                gp.Delete(name)


    gp.addmessage("Creating irr_str table")
    if gp.Exists("irr_str"):
        gp.delete("irr_str")
    gp.CreateTable(wksp + os.sep + "temp_wts.mdb", "irr_str")
    gp.AddField("irr_str","SW_WDID","text", "20")
    gp.AddField("irr_str","area","double")
    
    stats = "area sum"
    for rast in rasters:
        name = os.path.basename(rast)
        gp.AddField("irr_str","R_" + name,"double")
        stats = stats + ";R_" + name + " sum"
        if oroWorkSpace != "#":
            gp.AddField("irr_str","O_" + name,"double")
            stats = stats + ";O_" + name + " sum"

    wCur = gp.InsertCursor("irr_str")

    pRows = gp.SearchCursor("wtsTable")
    pRow = pRows.Next()
    while pRow:
        for i in range(9):
            cov = pRow.GetValue("first_SW_COV" + str(i+1))
            if cov > 0:
                wRow = wCur.NewRow()
                wRow.SW_WDID = pRow.GetValue("first_SW_WDID" + str(i+1))
                p_a = cov/100*pRow.GetValue("sum_Shape_Area")
                wRow.SetValue("area",pRow.GetValue("sum_Shape_Area"))
                for rast in rasters:
                    name = os.path.basename(rast)
                    wRow.SetValue("R_" + name,0)
                    if oroWorkSpace != "#":
                        wRow.SetValue("O_" + name,0)
                    if pRow.GetValue("R_" + name) > 0:
                        wRow.SetValue("R_" + name,pRow.GetValue("R_" + name)*p_a)
                        if oroWorkSpace != "#":
                            wRow.SetValue("O_" + name,pRow.GetValue("O_" + name)*p_a)
                wCur.InsertRow(wRow)

        pRow = pRows.Next()

    del wCur,wRow,pRows,pRow

    gp.addmessage("Creating summary statistics table")

    gp.Statistics("irr_str", "irr_str_sum", stats,"SW_WDID")

    time.sleep(5)    

    for rast in rasters:
        f = "sum_R_" + os.path.basename(rast)
        gp.CalculateField("irr_str_sum",f,"["+f+"]/[sum_area]")
        if oroWorkSpace != "#":
            f = "sum_O_" + os.path.basename(rast)
            gp.CalculateField("irr_str_sum",f,"["+f+"]/[sum_area]")

    gp.AddField("irr_str_sum","Total", "FLOAT")
    
    rows = gp.UpdateCursor("irr_str_sum")
    row = rows.Next()
    while row:
        desc = gp.Describe("irr_str_sum")
        fields = desc.Fields
        field = fields.Next()
        tot = 0
        while field:
            if field.Name[0:6] == "sum_R_" and row.GetValue(field.Name) > thresh:
                tot = tot + row.GetValue(field.Name)
            field = fields.Next()
        row.SetValue("TOTAL",tot)
        rows.UpdateRow(row)
        row = rows.Next()
    del row,rows,desc


    #Write Parameters and date to log file
    gp.addmessage("Writing output file")
    out = open(oFile,"w")

    out.write("#Date and Time |" + time.asctime()+ "\n")
    out.write("#Input Polygon Theme |" + theme + "\n")
    out.write("#Polygon ID Field |" + Zone_field + "\n")
    out.write("#Climate Weigths Rasters |" + rstrs + "\n")
    out.write("#Orograhpic Grids Workspace |" + oroWorkSpace + "\n")

    #gp.Workspace = wtsWorkSpace
    rows = gp.SearchCursor("irr_str_sum")
    row = rows.Next()
    while row:
        tot = 1.00
        desc = gp.Describe("irr_str_sum")
        fields = desc.Fields
        field = fields.Next()
        while field:
            if field.Name[0:6] == "sum_R_" and row.GetValue(field.Name) > thresh:
                x = round(row.GetValue(field.Name)/row.GetValue("TOTAL")*10000)/10000
                if (tot - x) < thresh:
                    x = tot
                tot = tot - x
                outstr = str(row.GetValue("SW_WDID")) + "," + field.Name.replace("sum_R_","") + "," + str(x)
                if oroWorkSpace != "#":
                    outstr = outstr + "," + str(round(row.GetValue(field.Name.replace("sum_R_","O_"))*10000)/10000)
                else:
                    outstr = outstr + ",1"

                out.write(outstr + "\n")
            field = fields.Next()
        row = rows.Next()

    del row,rows,orow,orows
        
    out.close()

except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
