# ---------------------------------------------------------------------------
# StateDGI_Intersect.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and 
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]

urf_file = sys.argv[2]
if urf_file != "#":
    urf_parts = int(sys.argv[3])

sw_agg_file = sys.argv[4]
if sw_agg_file != "#":
    sw_agg_parts = int(sys.argv[5])

sw_div_file = sys.argv[6]
if sw_div_file != "#":
    sw_div_parts = int(sys.argv[7])

can_div_file = sys.argv[8]
if can_div_file != "#":
    can_div_parts = int(sys.argv[9])

etz_table = sys.argv[10]

#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

try:
    #Write Parameters and date to log file
    #gp.addmessage("Writing log file")
    d = os.path.dirname(DGIDatabase)
    out = open(d + "/log.txt","w")

    out.write("#Date and Time\n")
    out.write(time.asctime()+ "\n")
    out.write("#Geodatabase\n")
    out.write(DGIDatabase + "\n")
    out.write("#URF file\n")
    out.write(urf_file + "\n")
    out.write("#sw_agg file\n")
    out.write(sw_agg_file + "\n")
    out.write("#sw_div file\n")
    out.write(sw_div_file + "\n")
    out.write("#canal_div file\n")
    out.write(can_div_file + "\n")

    if gp.Exists("irr_str_gw"):
        gp.delete("irr_str_gw")
    gp.CreateTable(DGIDatabase, "irr_str_gw")
    out.write('gp.CreateTable(DGIDatabase, "irr_str_gw")' + "\n")
    gp.AddField("irr_str_gw","PARCEL_ID","long", "12")
    gp.AddField("irr_str_gw","GW_WDID","text", "20")
    gp.AddField("irr_str_gw","GW_COV","double")
    out.write('added fields "irr_str_gw"' + "\n")
    
    if urf_file != "#":
        gp.addmessage("Loading URF/Agggreate Input file")
        f = open(urf_file)
        out.write("open(urf_file)" + "\n")
        wCur = gp.InsertCursor("irr_str_gw")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                wRow = wCur.NewRow()
                wRow.PARCEL_ID = id[urf_parts - 1]
                wRow.GW_WDID = id[0]
                wRow.GW_COV = 100
                wCur.InsertRow(wRow)
        f.close()
        out.write("close(urf_file)" + "\n")

        del wCur,wRow

    gp.addmessage("Creating SW Aggregate table")
    if gp.Exists("ms_str"):
        gp.delete("ms_str")
    gp.CreateTable(DGIDatabase, "ms_str")
    out.write('gp.CreateTable(DGIDatabase, "ms_str")' + "\n")
    gp.AddField("ms_str","MS_ID","text", "20")
    gp.AddField("ms_str","SW_WDID","text", "20")
    out.write('added fields to "ms_str")' + "\n")

    if gp.Exists("canal_aka"):
        gp.delete("canal_aka")
    gp.CreateTable(DGIDatabase, "canal_aka")
    gp.AddField("canal_aka","Canal_MSID","text", "20")
    gp.AddField("canal_aka","SW_WDID","text", "20")
    out.write('added fields to "canal_aka")' + "\n")

    if sw_agg_file != "#":
        wCur = gp.InsertCursor("ms_str")
        gp.addmessage("Loading SW Aggregate file")
        f = open(sw_agg_file)
        out.write("open(sw_agg_file)" + "\n")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                i = sw_agg_parts - 1
                while (i < len(id)):
                    wRow = wCur.NewRow()
                    wRow.MS_ID = id[0]
                    wRow.SW_WDID = id[i]
                    wCur.InsertRow(wRow)
                    i = i + 1
        f.close()
        out.write("close(sw_agg_file)" + "\n")

        del wCur,wRow
        
    if sw_agg_file != "#":
        wCur = gp.InsertCursor("canal_aka")
        f = open(sw_agg_file)
        out.write("open(sw_agg_file)" + "\n")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                i = sw_agg_parts - 1
                while (i < len(id)):
                    wRow = wCur.NewRow()
                    wRow.Canal_MSID = id[0]
                    wRow.SW_WDID = id[i]
                    wCur.InsertRow(wRow)
                    i = i + 1
        f.close()
        out.write("close(sw_agg_file)" + "\n")

        del wCur,wRow
        
    if sw_div_file != "#":
        wCur = gp.InsertCursor("ms_str")
        gp.addmessage("Loading SW Diversion Systems Input file")
        f = open(sw_div_file)
        out.write("open(sw_div_file)" + "\n")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                i = sw_div_parts - 1
                while (i < len(id)):
                    wRow = wCur.NewRow()
                    wRow.MS_ID = id[0]
                    wRow.SW_WDID = id[i]
                    wCur.InsertRow(wRow)
                    i = i + 1
        f.close()
        out.write("close(sw_div_file)" + "\n")

        del wCur,wRow
        
    if can_div_file != "#":
        wCur = gp.InsertCursor("canal_aka")
        gp.addmessage("Loading Canal Diversion System file")
        f = open(can_div_file)
        out.write("open(canal_div_file)" + "\n")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                i = can_div_parts - 1
                while (i < len(id)):
                    wRow = wCur.NewRow()
                    wRow.Canal_MSID = id[0]
                    wRow.SW_WDID = id[i]
                    wCur.InsertRow(wRow)
                    i = i + 1
        f.close()
        out.write("close(canal_div_file)" + "\n")

        del wCur,wRow

#commented out this area to keep cover_id from being deleted
    #if gp.Exists("cover_id"):
     #   gp.delete("cover_id")
    gp.CreateTable(DGIDatabase, "cover_id")
    out.write('gp.CreateTable(DGIDatabase, "cover_id")' + "\n")
    gp.AddField("cover_id","ground_cover_id","text", "40")
    gp.AddField("cover_id","etz_id","text", "40")
    out.write('added fields "cover_id"' + "\n")

    if etz_table != "#":
        wCur = gp.InsertCursor("cover_id")
        gp.addmessage("Loading ET Zone Alias file")
        f = open(etz_table)
        out.write('open(etz_table)' + "\n")
        for line in f.readlines():
            if not line.startswith("#"):
                id = line.strip().split(",")
                wRow = wCur.NewRow()
                wRow.ground_cover_id = id[0]
                wRow.etz_id = id[1]
                wCur.InsertRow(wRow)
        f.close()
        out.write('close(etz_table)' + "\n")


    gp.compact(DGIDatabase)

    out.close()

except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
    