# ---------------------------------------------------------------------------
# StateDGI_Intersect.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and 
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]
model_output = sys.argv[2]
irr_parcel = sys.argv[3]
irr_parcel_wt_att = sys.argv[4]
irr_parcel_wt_val = sys.argv[5]
irr_parcel_l_att = sys.argv[6]
irr_parcel_l_val = sys.argv[7]
riminflow = sys.argv[8]
riminflow_wt_att = sys.argv[9]
riminflow_wt_val = sys.argv[10]
riminflow_l_att = sys.argv[11]
riminflow_l_val = sys.argv[12]
ground_cover = sys.argv[13]
ground_cover_wt_att = sys.argv[14]
ground_cover_wt_val = sys.argv[15]
ground_cover_l_att = sys.argv[16]
ground_cover_l_val = sys.argv[17]
canals = sys.argv[18]
canals_wt_att = sys.argv[19]
canals_wt_val = sys.argv[20]
canals_l_att = sys.argv[21]
canals_l_val = sys.argv[22]
irr_wells = sys.argv[23]
layer = sys.argv[24]
weight = sys.argv[25]


#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

if model_output != "#":
    if gp.Exists("model"):
        gp.delete("model")
    gp.addmessage("Loading Model shapefile")
    gp.CopyFeatures(model_output, DGIDatabase + os.sep + "model")
    
#For the "parcels" shapefile, create 2 new fields to identify centroid
#locations. Calculate centroids for all polygons in shapefile and place
#coordinates in the appropriate field.

if irr_parcel != "#":
    if gp.Exists("parcels"):
        gp.delete("parcels")
    gp.addmessage("Loading parcel shapefile")
    gp.CopyFeatures(irr_parcel, DGIDatabase + os.sep + "parcels")

    gp.addmessage("Creating well_parcel table")
    gp.CreateTable(DGIDatabase, "well_parcel")
    gp.AddField("well_parcel","PARCEL_ID","long", "12")
    gp.AddField("well_parcel","WELL_ID","text", "20")
    gp.AddField("well_parcel","CLASS","text", "20")
    wCur = gp.InsertCursor("well_parcel")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.Next()
    while pRow:
        for i in range(20):
            if len(pRow.GetValue("GW_ID" + str(i+1))) > 2:
                wRow = wCur.NewRow()
                wRow.PARCEL_ID = pRow.PARCEL_ID
                wRow.WELL_ID = pRow.GetValue("GW_ID" + str(i+1))
                wRow.CLASS = pRow.GetValue("GW_CLASS" + str(i+1))
                wCur.InsertRow(wRow)
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow        

    gp.addmessage("Creating irr_str table")
    gp.CreateTable(DGIDatabase, "irr_str")
    gp.AddField("irr_str","PARCEL_ID","long", "12")
    gp.AddField("irr_str","SW_WDID","text", "20")
    gp.AddField("irr_str","SW_COV","long", "4")
    wCur = gp.InsertCursor("irr_str")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.Next()
    while pRow:
        for i in range(9):
            if pRow.GetValue("SW_COV" + str(i+1)) > 0:
                wRow = wCur.NewRow()
                wRow.PARCEL_ID = pRow.PARCEL_ID
                wRow.SW_WDID = pRow.GetValue("SW_WDID" + str(i+1))
                wRow.SW_COV = pRow.GetValue("SW_COV" + str(i+1))
                wCur.InsertRow(wRow)
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow        

    #Create Centroids
    gp.addmessage("Computing parcel centroids")
    gp.CreateFeatureClass(DGIDatabase,"parcel_centers","point","#","#","#",gp.Describe("parcels").SpatialReference)    
    gp.addfield ("parcel_centers", "PARCEL_ID", "long", "12")
    cenCur = gp.InsertCursor("parcel_centers")

    p = gp.CreateObject("point")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.next()
    while pRow:
        pp = string.split(pRow.Shape.Centroid," ")
        p.x = string.atof(pp[0])
        p.y = string.atof(pp[1])
        cenRow = cenCur.NewRow()
        cenRow.Shape = p
        cenRow.SetValue("PARCEL_ID",pRow.GetValue("PARCEL_ID"))
        cenCur.InsertRow(cenRow)
        pRow = pRows.next()

    del pRow,pRows,cenCur,cenRow,p


if riminflow != "#":
    if gp.Exists("riminflow"):
        gp.delete("riminflow")
    gp.addmessage("Loading Rim Inflows shapefile")
    gp.CopyFeatures(riminflow, DGIDatabase + os.sep + "riminflow")
    
if ground_cover != "#":
    if gp.Exists("ground_cover"):
        gp.delete("ground_cover")
    gp.addmessage("Loading Groundcover shapefile")
    gp.CopyFeatures(ground_cover, DGIDatabase + os.sep + "ground_cover")
   
if canals != "#":
    if gp.Exists("canals"):
        gp.delete("canals")
    gp.addmessage("Loading Canals shapefile")
    gp.CopyFeatures(canals, DGIDatabase + os.sep + "canals")
    
if irr_wells != "#":
    if gp.Exists("wells"):
        gp.delete("wells")
    gp.addmessage("Loading Wells shapefile")
    gp.CopyFeatures(irr_wells, DGIDatabase + os.sep + "wells")

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("info_wells"):
        gp.delete("info_wells")
    if gp.Exists("no_info_wells"):
        gp.delete("no_info_wells")
    if gp.Exists("Est_wells"):
        gp.delete("Est_wells")
    gp.CreateTable(DGIDatabase, "Est_wells")
    gp.AddField("Est_wells","WELL_ID","text", "20")
    gp.AddField("Est_wells","EST_ID","text", "20")
 
    gp.MakeFeatureLayer("wells", "wells_lyr")
    gp.SelectLayerByAttribute("wells_lyr", 'new_selection', "BPERF > 0")
    gp.SelectLayerByAttribute("wells_lyr", 'add_to_selection', "DEPTH > 0")
    gp.SelectLayerByAttribute("wells_lyr", 'add_to_selection', "AQUIFER1 <> ''")
    gp.CopyFeatures("wells_lyr", "info_wells")
    gp.SelectLayerByAttribute("wells_lyr", 'switch_selection', "")
    gp.CopyFeatures("wells_lyr", "no_info_wells")
    #buffer two miles
    if gp.Exists("no_info_buffer"):
        gp.delete("no_info_buffer")
    gp.buffer("no_info_wells", "no_info_buffer", 3218)
    if gp.Exists("wells_intersect"):
        gp.delete("wells_intersect")
    gp.Intersect_analysis("info_wells;no_info_buffer", "wells_intersect")

    tabCur = gp.InsertCursor("Est_wells")
    
    wRows = gp.SearchCursor("no_info_wells")
    wRow = wRows.next()
    while wRow:
        pRows = gp.SearchCursor("wells_intersect", "WELL_ID_1 = '" + wRow.GetValue("WELL_ID") + "'")
        pRow = pRows.next()
        dist = 999999
        while pRow:
            wp = wRow.Shape.GetPart(0)
            pp = pRow.Shape.GetPart(0)
            dx = wp.x - pp.x
            dy = wp.y - pp.y
            d = math.sqrt(dx*dx + dy*dy)
            if (d < dist):
                id = pRow.GetValue("WELL_ID")
                dist = d
            pRow = pRows.next()

        if (dist < 99999):            
            tRow = tabCur.NewRow()
            tRow.SetValue("WELL_ID",wRow.GetValue("WELL_ID"))
            tRow.SetValue("EST_ID",id)
            tabCur.InsertRow(tRow)

        wRow = wRows.next()
    
    del wRow,wRows,tRow,tabCur

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("info_wells"):
        gp.delete("info_wells")
    if gp.Exists("no_info_wells"):
        gp.delete("no_info_wells")
    if gp.Exists("no_info_buffer"):
        gp.delete("no_info_buffer")
    if gp.Exists("wells_intersect"):
        gp.delete("wells_intersect")
    
if layer != "#":
    if gp.Exists("layer"):
        gp.delete("layer")
    gp.FeatureClassToGeodatabase(layer, DGIDatabase)
    temp = os.path.basename(layer)
    tmp = temp.split(".")
    gp.rename(tmp[0], "layer")

if weight != "#":
    if gp.Exists("weight"):
        gp.delete("weight")
    gp.FeatureClassToGeodatabase(weight, DGIDatabase)
    temp = os.path.basename(weight)
    tmp = temp.split(".")
    gp.rename(tmp[0], "weight")
    
#Add Weight values to shapefiles.
if irr_parcel != "#":
    gp.addmessage("Adding parcel weights")
    if irr_parcel_wt_val != "#":
        gp.addfield("parcels", "Weight", "double", "6", "2")
        gp.calculatefield ("parcels", "Weight",irr_parcel_wt_val)
    elif irr_parcel_wt_att != "#":
        if string.upper(irr_parcel_wt_att) != "WEIGHT":
            gp.addfield ("parcels", "Weight", "double", "12", "2")
            gp.calculatefield ("parcels", "Weight",irr_parcel_wt_att)
    elif gp.Exists("weight"):
        if gp.Exists("parcels_tmp"):
            gp.delete("parcels_tmp")
        gp.Intersect_analysis("parcels;weight", "parcels_tmp",)
        gp.delete("parcels")
        gp.rename("parcels_tmp","parcels")
    else:
        gp.addmessage("Warning - No value for Parcel Weights!")

if riminflow != "#":
    gp.addmessage("Adding riminflow weights")
    if riminflow_wt_val != "#":
        gp.addfield("riminflow", "Weight", "double", "6", "2")
        gp.calculatefield ("riminflow", "Weight",riminflow_wt_val)
    elif riminflow_wt_att != "#":
        if string.upper(riminflow_wt_att) != "WEIGHT":
            gp.addfield ("riminflow", "Weight", "double", "12", "2")
            gp.calculatefield ("riminflow", "Weight",riminflow_wt_att)
    elif gp.Exists("riminflow"):
        if gp.Exists("riminflow_tmp"):
            gp.delete("riminflow_tmp")
        gp.Intersect_analysis("riminflow;weight", "riminflow_tmp",)
        gp.delete("riminflow")
        gp.rename("riminflow_tmp","riminflow")
    else:
        gp.addmessage("Warning - No value for Riminflow Weights!")

if canals != "#":
    gp.addmessage("Adding canals weights")
    if canals_wt_val != "#":
        gp.addfield("canals", "Weight", "double", "6", "2")
        gp.calculatefield ("canals", "Weight",canals_wt_val)
    elif canals_wt_att != "#":
        if string.upper(canals_wt_att) != "WEIGHT":
            gp.addfield ("canals", "Weight", "double", "12", "2")
            gp.calculatefield ("canals", "Weight",canals_wt_att)
    elif gp.Exists("canals"):
        if gp.Exists("canals_tmp"):
            gp.delete("canals_tmp")
        gp.Intersect_analysis("canals;weight", "canals_tmp",)
        gp.delete("canals")
        gp.rename("canals_tmp","canals")
    else:
        gp.addmessage("Warning - No value for Canal Weights!")

if ground_cover != "#":
    gp.addmessage("Adding ground_cover weights")
    if ground_cover_wt_val != "#":
        gp.addfield("ground_cover", "Weight", "double", "6", "2")
        gp.calculatefield ("ground_cover", "Weight",ground_cover_wt_val)
    elif ground_cover_wt_att != "#":
        if string.upper(ground_cover_wt_att) != "WEIGHT":
            gp.addfield ("ground_cover", "Weight", "double", "12", "2")
            gp.calculatefield ("ground_cover", "Weight",canals_wt_att)
    elif gp.Exists("ground_cover"):
        if gp.Exists("canals_tmp"):
            gp.delete("canals_tmp")
        gp.Intersect_analysis("ground_cover;weight", "ground_cover_tmp",)
        gp.delete("ground_cover")
        gp.rename("ground_cover_tmp","ground_cover")
    else:
        gp.addmessage("Warning - No value for Ground Cover Weights!")



if irr_parcel != "#":
    gp.addmessage("Adding parcel layers")
    if irr_parcel_l_val != "#":
        gp.addfield ("parcels", "Layer", "short", "4")
        gp.calculatefield ("parcels", "Layer",irr_parcel_l_val)
    elif irr_parcel_l_att != "#":
        if string.upper(irr_parcel_l_att) != "LAYER":
            gp.addfield ("parcels", "Layer", "short", "4")
            gp.calculatefield ("parcels", "Weight",irr_parcel_l_att)
    elif gp.Exists("layer"):
        if gp.Exists("parcels_tmp"):
            gp.delete("parcels_tmp")
        gp.Intersect_analysis("parcels;layer", "parcels_tmp",)
        gp.delete("parcels")
        gp.rename("parcels_tmp","parcels")
    else:
        gp.addmessage("Warning - No value for Model Layer!")


if riminflow != "#":
    gp.addmessage("Adding riminflow layers")
    if riminflow_l_val != "#":
        gp.addfield ("riminflow", "Layer", "short", "4")
        gp.calculatefield ("riminflow", "Layer",riminflow_l_val)
    elif riminflow_l_att != "#":
        if string.upper(riminflow_l_att) != "LAYER":
            gp.addfield ("riminflow", "Layer", "short", "4")
            gp.calculatefield ("riminflow", "Weight",riminflow_l_att)
    elif gp.Exists("layer"):
        if gp.Exists("riminflow_tmp"):
            gp.delete("riminflow_tmp")
        gp.Intersect_analysis("riminflow;layer", "riminflow_tmp",)
        gp.delete("riminflow")
        gp.rename("riminflow_tmp","riminflow")
    else:
        gp.addmessage("Warning - No value for Riminflow Layer!")

if canals != "#":
    gp.addmessage("Adding canal layers")
    if canals_l_val != "#":
        gp.addfield ("canals", "Layer", "short", "4")
        gp.calculatefield ("canals", "Layer",canals_l_val)
    elif canals_l_att != "#":
        if string.upper(canals_l_att) != "LAYER":
            gp.addfield ("canals", "Layer", "short", "4")
            gp.calculatefield ("canals", "Layer",canals_l_att)
    elif gp.Exists("layer"):
        if gp.Exists("canals_tmp"):
            gp.delete("canals_tmp")
        gp.Intersect_analysis("canals;layer", "canals_tmp",)
        gp.delete("canals")
        gp.rename("canals_tmp","canals")
    else:
        gp.addmessage("Warning - No value for Canals Layer!")

if ground_cover != "#":
    gp.addmessage("Adding ground cover layers")
    if ground_cover_l_val != "#":
        gp.addfield ("ground_cover", "Layer", "short", "4")
        gp.calculatefield ("ground_cover", "Layer",ground_cover_l_val)
    elif ground_cover_l_att != "#":
        if string.upper(ground_cover_l_att) != "LAYER":
            gp.addfield ("ground_cover", "Layer", "short", "4")
            gp.calculatefield ("ground_cover", "Layer",ground_cover_l_att)
    elif gp.Exists("layer"):
        if gp.Exists("ground_cover_tmp"):
            gp.delete("ground_cover_tmp")
        gp.Intersect_analysis("ground_cover;layer", "ground_cover_tmp",)
        gp.delete("ground_cover")
        gp.rename("ground_cover_tmp","ground_cover")
    else:
        gp.addmessage("Warning - No value for ground cover Layer!")



                
#Perform Model Intersection.
if gp.exists(model_output):
    gp.addmessage("Intersecting Input Shapefiles with Model")
    if gp.exists("parcels"):
        gp.addmessage("Intersecting parcels")
        if gp.Exists("parcel_cells"):
            gp.delete("parcel_cells")
        gp.Intersect_analysis("model ; parcels ", "parcel_cells")
        #gp.addfield ("parcel_cells", "ACRES", "double", "12", "2")
        #gp.calculatefield ("parcel_cells", "ACRES","[Shape_Area] / 4046.95")
        
    if gp.exists("parcel_centers"):
        gp.addmessage("Intersecting parcel centers")
        if gp.Exists("parcel_center_cells"):
            gp.delete("parcel_center_cells")
        gp.Intersect_analysis("model ; parcel_centers ", "parcel_center_cells")
        
    if gp.exists("riminflow"):
        gp.addmessage("Intersecting riminflow")
        if gp.Exists("riminflow_cells"):
            gp.delete("riminflow_cells")
        gp.Intersect_analysis("model ; riminflow ", "riminflow_cells")
        #gp.addfield ("riminflow_cells", "ACRES", "double", "12", "2")
        #gp.calculatefield ("riminflow_cells", "ACRES","[Shape_Area] / 4046.95")
        
    if gp.exists("ground_cover"):
        gp.addmessage("Intersecting ground_cover")
        if gp.Exists("ground_cover_cells"):
            gp.delete("ground_cover_cells")
        gp.Intersect_analysis("model ; ground_cover ", "ground_cover_cells")
        #gp.addfield ("ground_cover_cells", "ACRES", "double", "12", "2")
        #gp.calculatefield ("ground_cover_cells", "ACRES","[Shape_Area] / 4046.95")
        
    if gp.exists("canals"):
        gp.addmessage("Intersecting canals")
        if gp.Exists("canal_cells"):
            gp.delete("canal_cells")
        gp.Intersect_analysis("model ; canals ", "canal_cells")

    if gp.exists("wells"):
        gp.addmessage("Intersecting wells")
        if gp.Exists("wells_cells"):
            gp.delete("wells_cells")
        gp.Intersect_analysis("model ; wells ", "wells_cells")



gp.compact(DGIDatabase)

#Write Parameters and date to log file
gp.addmessage("Writing log file")
d = os.path.dirname(DGIDatabase)
out = open(d + "/log.txt","w")

out.write("#Date and Time\n")
out.write(time.asctime()+ "\n")
out.write("#Geodatabase\n")
out.write(DGIDatabase + "\n")
out.write("#Model Shapefile\n")
out.write(model_output + "\n")
out.write("#Irrigated Acres Shapefile\n")
out.write(irr_parcel + "\n")
out.write("#Rim Inflows Shapefile\n")
out.write(riminflow + "\n")
out.write("#Ground Cover Shapefile\n")
out.write(ground_cover + "\n")
out.write("#Canals Shapefile\n")
out.write(canals + "\n")
out.write("#Irrigation Wells Shapefile\n")
out.write(irr_wells + "\n")

out.close()