# ---------------------------------------------------------------------------
# StateDGI_Intersect.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and 
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]
irr_parcel = sys.argv[2]
#irr_parcel_wt_val = sys.argv[3]
weight = sys.argv[3]

#urf_file = sys.argv[5]

#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

#Write Parameters and date to log file
gp.addmessage("Writing log file")
d = os.path.dirname(DGIDatabase)
out = open(d + "/log.txt","w")

out.write("#Date and Time\n")
out.write(time.asctime()+ "\n")
out.write("#Geodatabase\n")
out.write(DGIDatabase + "\n")
out.write("#Irrigated Acres Shapefile\n")
out.write(irr_parcel + "\n")
#out.write("#Irrigated weight value\n")
#out.write(irr_parcel_wt_val + "\n")
out.write("#Irrigated weight Shapefile\n")
out.write(weight + "\n")

#For the "parcels" shapefile, create 2 new fields to identify centroid
#locations. Calculate centroids for all polygons in shapefile and place
#coordinates in the appropriate field.

if irr_parcel != "#":
    if gp.Exists("parcels"):
        gp.delete("parcels")
    gp.addmessage("Loading parcel shapefile")
    gp.CopyFeatures(irr_parcel, DGIDatabase + os.sep + "parcels")
    out.write('gp.CopyFeatures(irr_parcel, DGIDatabase + os.sep + "parcels")' + "\n")

    gp.addmessage("Creating well_parcel table")
    if gp.Exists("well_parcel"):
        gp.delete("well_parcel")
    gp.CreateTable(DGIDatabase, "well_parcel")
    out.write('gp.CreateTable(DGIDatabase, "well_parcel")' + "\n")
    gp.AddField("well_parcel","PARCEL_ID","long", "12")
    gp.AddField("well_parcel","WELL_ID","text", "20")
    gp.AddField("well_parcel","TYPE","text", "20")
    gp.AddField("well_parcel","CLASS","text", "20")
    out.write('added fields "well_parcel"' + "\n")
    wCur = gp.InsertCursor("well_parcel")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.Next()
    while pRow:
        for i in range(20):
            if len(pRow.GetValue("GW_ID" + str(i+1))) > 2:
                wRow = wCur.NewRow()
                wRow.PARCEL_ID = pRow.PARCEL_ID
                wRow.WELL_ID = pRow.GetValue("GW_ID" + str(i+1))
                wRow.TYPE = pRow.GetValue("GW_TYPE" + str(i+1))
                #Reset Type 9 to type 4
                if (pRow.GetValue("GW_TYPE" + str(i+1)) == "9"):
                    wRow.TYPE = "4"
                wRow.CLASS = pRow.GetValue("GW_CLASS" + str(i+1))
                wCur.InsertRow(wRow)
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow        

    gp.addmessage("Creating irr_str table")
    if gp.Exists("irr_str_sw"):
        gp.delete("irr_str_sw")
    if gp.Exists("irr_str_gw"):
        gp.delete("irr_str_gw")
    gp.CreateTable(DGIDatabase, "irr_str_sw")
    out.write('gp.CreateTable(DGIDatabase, "irr_str_sw")' + "\n")
    gp.AddField("irr_str_sw","PARCEL_ID","long", "12")
    gp.AddField("irr_str_sw","SW_WDID","text", "20")
    gp.AddField("irr_str_sw","SW_COV","double")

    gp.CreateTable(DGIDatabase, "irr_str_gw")
    out.write('gp.CreateTable(DGIDatabase, "irr_str_gw")' + "\n")
    gp.AddField("irr_str_gw","PARCEL_ID","long", "12")
    gp.AddField("irr_str_gw","GW_WDID","text", "20")
    gp.AddField("irr_str_gw","GW_COV","double")
    out.write('added fields "irr_str_gw"' + "\n")

    wCur = gp.InsertCursor("irr_str_sw")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.Next()
    while pRow:
        for i in range(9):
            if pRow.GetValue("SW_COV" + str(i+1)) > 0:
                wRow = wCur.NewRow()
                wRow.PARCEL_ID = pRow.PARCEL_ID
                wRow.SW_WDID = pRow.GetValue("SW_WDID" + str(i+1))
                wRow.SW_COV = pRow.GetValue("SW_COV" + str(i+1))
                wCur.InsertRow(wRow)
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow

    #gp.addmessage("Creating ms_str table")
    if gp.Exists("ms_str"):
        gp.delete("ms_str")
    gp.CreateTable(DGIDatabase, "ms_str")
    gp.AddField("ms_str","MS_ID","text", "20")
    gp.AddField("ms_str","SW_WDID","text", "20")        

    #Create Centroids
    gp.addmessage("Computing parcel centroids")
    if gp.Exists("parcel_centers"):
        gp.delete("parcel_centers")
    gp.CreateFeatureClass(DGIDatabase,"parcel_centers","point","#","#","#",gp.Describe("parcels").SpatialReference)    
    out.write('gp.CreateFeatureClass(DGIDatabase,"parcel_centers","point","#","#","#",gp.Describe("parcels").SpatialReference)' + "\n")
    gp.addfield ("parcel_centers", "PARCEL_ID", "long", "12")
    cenCur = gp.InsertCursor("parcel_centers")

    p = gp.CreateObject("point")
    pRows = gp.SearchCursor("parcels")
    pRow = pRows.next()
    while pRow:
        pp = string.split(pRow.Shape.Centroid," ")
        p.x = string.atof(pp[0])
        p.y = string.atof(pp[1])
        cenRow = cenCur.NewRow()
        cenRow.Shape = p
        cenRow.SetValue("PARCEL_ID",pRow.GetValue("PARCEL_ID"))
        cenCur.InsertRow(cenRow)
        pRow = pRows.next()

    del pRow,pRows,cenCur,cenRow,p
  

if weight != "#":
    if gp.Exists("weight"):
        gp.delete("weight")
    gp.FeatureClassToGeodatabase(weight, DGIDatabase)
    out.write('gp.FeatureClassToGeodatabase(weight, DGIDatabase)' + "\n")
    temp = os.path.basename(weight)
    tmp = temp.split(".")
    gp.rename(tmp[0], "weight")
    
#Add Weight values to shapefiles.
if irr_parcel != "#":
    gp.addmessage("Adding parcel weights")
    if weight != "#":
        if gp.Exists("parcels_tmp"):
            gp.delete("parcels_tmp")
        gp.Intersect_analysis("parcels;weight", "parcels_tmp",)
        out.write('gp.Intersect_analysis("parcels;weight", "parcels_tmp",)' + "\n")
        gp.delete("parcels")
        gp.rename("parcels_tmp","parcels")
    else:
        gp.addfield("parcels", "Weight", "double", "6", "2")
        gp.calculatefield ("parcels", "Weight",1)
        out.write('gp.calculatefield ("parcels", "Weight",irr_parcel_wt_val)' + "\n")



#Perform Model Intersection.
if gp.exists("model"):
    gp.addmessage("Intersecting Input Shapefiles with Model")
    if gp.exists("parcels"):
        gp.addmessage("Intersecting parcels")
        if gp.Exists("parcel_cells"):
            gp.delete("parcel_cells")
        gp.Intersect_analysis("model ; parcels ", "parcel_cells")
        out.write('gp.Intersect_analysis("model ; parcels ", "parcel_cells")' + "\n")
        
    if gp.exists("parcel_centers"):
        gp.addmessage("Intersecting parcel centers")
        if gp.Exists("parcel_center_cells"):
            gp.delete("parcel_center_cells")
        gp.Intersect_analysis("model ; parcel_centers ", "parcel_center_cells")
        out.write('gp.Intersect_analysis("model ; parcel_centers ", "parcel_center_cells")' + "\n")
        

gp.compact(DGIDatabase)

out.close()