# ---------------------------------------------------------------------------
# StateDGI_Model.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and 
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]
model_output = sys.argv[2]


#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

#Write Parameters and date to log file
gp.addmessage("Writing log file")
d = os.path.dirname(DGIDatabase)
out = open(d + "/log.txt","w")

out.write("#Date and Time\n")
out.write(time.asctime()+ "\n")
out.write("#Geodatabase\n")
out.write(DGIDatabase + "\n")
out.write("#Model Shapefile\n")
out.write(model_output + "\n")

if model_output != "#":
    if gp.Exists("model"):
        gp.delete("model")
    gp.addmessage("Loading Model shapefile")
    gp.CopyFeatures(model_output, DGIDatabase + os.sep + "model")
    out.write('gp.CopyFeatures(model_output, DGIDatabase + os.sep + "model")' + "\n")

    gp.addmessage("Creating model_layer table")
    if gp.Exists("model_layer"):
        gp.delete("model_layer")
    gp.CreateTable(DGIDatabase, "model_layer")
    out.write('gp.CreateTable(DGIDatabase, "model_layer")' + "\n")
    gp.AddField("model_layer","ROW","long", "12")
    gp.AddField("model_layer","COL","long", "12")
    gp.AddField("model_layer","LAYER","long", "12")
    gp.AddField("model_layer","TOP","double", "20")
    gp.AddField("model_layer","BOTTOM","double", "20")
    out.write('Added Fields' + "\n")
    wCur = gp.InsertCursor("model_layer")
    pRows = gp.SearchCursor("model")
    pRow = pRows.Next()
    f = gp.ListFields("model")
    ff = f.Next()
    l = 0
    while ff:
        if ff.Name[:7] == "BOT_LYR":
            l = l + 1
        ff = f.Next()
    del f,ff    

    while pRow:
        for i in range(l):
            #gp.addmessage("BOT_LYR" + str(i+1))
            wRow = wCur.NewRow()
            wRow.ROW = pRow.ROW
            wRow.COL = pRow.COL
            wRow.LAYER = i+1
            if (i == 0):
                wRow.TOP = 0
            else:
                wRow.TOP = pRow.GND_ELEV - pRow.GetValue("BOT_LYR" + str(i))
            wRow.BOTTOM = pRow.GND_ELEV - pRow.GetValue("BOT_LYR" + str(i+1))
            wCur.InsertRow(wRow)

                    
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow        
    
    
#Perform Model Intersection.
if gp.exists("model"):
    gp.addmessage("Intersecting Input Shapefiles with Model")
    if gp.exists("parcels"):
        gp.addmessage("Intersecting parcels")
        if gp.Exists("parcel_cells"):
            gp.delete("parcel_cells")
        gp.Intersect_analysis("model ; parcels ", "parcel_cells")
        out.write('gp.Intersect_analysis("model ; parcels ", "parcel_cells")' + "\n")
        #gp.addfield ("parcel_cells", "ACRES", "double", "12", "2")
        #gp.calculatefield ("parcel_cells", "ACRES","[Shape_Area] / 4046.95")
        
    if gp.exists("parcel_centers"):
        gp.addmessage("Intersecting parcel centers")
        if gp.Exists("parcel_center_cells"):
            gp.delete("parcel_center_cells")
        gp.Intersect_analysis("model ; parcel_centers ", "parcel_center_cells")
        out.write('gp.Intersect_analysis("model ; parcel_centers ", "parcel_center_cells")' + "\n")
        
    if gp.exists("riminflow"):
        gp.addmessage("Intersecting riminflow")
        if gp.Exists("riminflow_cells"):
            gp.delete("riminflow_cells")
        gp.Intersect_analysis("model ; riminflow ", "riminflow_cells")
        out.write('gp.Intersect_analysis("model ; riminflow ", "riminflow_cells")' + "\n")
        
    if gp.exists("ground_cover"):
        gp.addmessage("Intersecting ground_cover")
        if gp.Exists("ground_cover_cells"):
            gp.delete("ground_cover_cells")
        gp.Intersect_analysis("model ; ground_cover ", "ground_cover_cells")
        out.write('gp.Intersect_analysis("model ; ground_cover ", "ground_cover_cells")' + "\n")
        
    if gp.exists("canals"):
        gp.addmessage("Intersecting canals")
        if gp.Exists("canal_cells"):
            gp.delete("canal_cells")
        gp.Intersect_analysis("model ; canals ", "canal_cells")
        out.write('gp.Intersect_analysis("model ; canals ", "canal_cells")' + "\n")

    if gp.exists("wells"):
        gp.addmessage("Intersecting wells")
        if gp.Exists("wells_cells"):
            gp.delete("wells_cells")
        gp.Intersect_analysis("model ; wells ", "wells_cells")
        out.write('gp.Intersect_analysis("model ; wells ", "wells_cells")' + "\n")
        gp.addfield("wells_cells","DUPLICATE","LONG")
        gp.CalculateField("wells_cells","DUPLICATE",0)



gp.compact(DGIDatabase)


out.close()