# ---------------------------------------------------------------------------
# StateDGI_Intersect.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and 
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]
riminflow = sys.argv[2]
#riminflow_wt_val = sys.argv[3]
riminflow_wt_att = sys.argv[3]
weight = sys.argv[4]
riminflow_l_val = sys.argv[5]
#riminflow_l_att = sys.argv[7]
layer = sys.argv[6]


#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

#Write Parameters and date to log file
gp.addmessage("Writing log file")
d = os.path.dirname(DGIDatabase)
out = open(d + "/log.txt","w")

out.write("#Date and Time\n")
out.write(time.asctime()+ "\n")
out.write("#Geodatabase\n")
out.write(DGIDatabase + "\n")
out.write("#Rim Inflows Shapefile\n")
out.write(riminflow + "\n")
out.write("#Rim Inflows Weight Attribute\n")
out.write(riminflow_wt_att + "\n")
out.write("#Rim Inflows Weight Shapefile\n")
out.write(weight + "\n")
out.write("#Rim Inflows Layer Shapefile\n")
out.write(layer + "\n")

if riminflow != "#":
    if gp.Exists("riminflow"):
        gp.delete("riminflow")
    gp.addmessage("Loading Rim Inflows shapefile")
    gp.CopyFeatures(riminflow, DGIDatabase + os.sep + "riminflow")
    out.write('gp.CopyFeatures(riminflow, DGIDatabase + os.sep + "riminflow")' + "\n")
    
if layer != "#":
    if gp.Exists("layer"):
        gp.delete("layer")
    gp.FeatureClassToGeodatabase(layer, DGIDatabase)
    out.write('gp.FeatureClassToGeodatabase(layer, DGIDatabase)' + "\n")
    temp = os.path.basename(layer)
    tmp = temp.split(".")
    gp.rename(tmp[0], "layer")

if weight != "#":
    if gp.Exists("weight"):
        gp.delete("weight")
    gp.FeatureClassToGeodatabase(weight, DGIDatabase)
    out.write('gp.FeatureClassToGeodatabase(weight, DGIDatabase)' + "\n")
    temp = os.path.basename(weight)
    tmp = temp.split(".")
    gp.rename(tmp[0], "weight")
    
if riminflow != "#":
    gp.addmessage("Adding riminflow weights")
    if riminflow_wt_att != "#":
        if string.upper(riminflow_wt_att) != "WEIGHT":
            gp.addfield ("riminflow", "Weight", "double", "12", "2")
            gp.calculatefield ("riminflow", "Weight",riminflow_wt_att)
            out.write('gp.calculatefield ("riminflow", "Weight",riminflow_wt_att)' + "\n")
    elif weight != "#":
        if gp.Exists("riminflow_tmp"):
            gp.delete("riminflow_tmp")
        gp.Intersect_analysis("riminflow;weight", "riminflow_tmp",)
        out.write('gp.Intersect_analysis("riminflow;weight", "riminflow_tmp",)' + "\n")
        gp.delete("riminflow")
        gp.rename("riminflow_tmp","riminflow")
    else:
        gp.addfield("riminflow", "Weight", "double", "6", "2")
        gp.calculatefield ("riminflow", "Weight",riminflow_wt_val)
        out.write('gp.calculatefield ("riminflow", "Weight",riminflow_wt_val)' + "\n")

if riminflow != "#":
    gp.addmessage("Adding riminflow layers")
    if layer != "#":
        if gp.Exists("riminflow_tmp"):
            gp.delete("riminflow_tmp")
        gp.Intersect_analysis("riminflow;layer", "riminflow_tmp",)
        out.write('gp.Intersect_analysis("riminflow;layer", "riminflow_tmp",)' + "\n")
        gp.delete("riminflow")
        gp.rename("riminflow_tmp","riminflow")
    else:
        gp.addfield ("riminflow", "Layer", "short", "4")
        gp.calculatefield ("riminflow", "Layer",riminflow_l_val)
        out.write('gp.calculatefield ("riminflow", "Layer",riminflow_l_val)' + "\n")

               
#Perform Model Intersection.
if gp.exists("model"):
    gp.addmessage("Intersecting Input Shapefiles with Model")
        
    if gp.exists("riminflow"):
        gp.addmessage("Intersecting riminflow")
        if gp.Exists("riminflow_cells"):
            gp.delete("riminflow_cells")
        gp.Intersect_analysis("model ; riminflow ", "riminflow_cells")
        out.write('gp.Intersect_analysis("model ; riminflow ", "riminflow_cells")' + "\n")

gp.compact(DGIDatabase)


out.close()