# ---------------------------------------------------------------------------
# StateDGI_Wells.py
# Test script fot the StateDGI tool.
#Intersect parameter shapefiles with RGDSS Task 3.1 model output and Wells
#load into geodatabase with assigned name.
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
DGIDatabase = sys.argv[1]
irr_wells = sys.argv[2]


#Assign work environment
#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
gp.Workspace = DGIDatabase
gp.overwriteoutput = 1

#Write Parameters and date to log file
gp.addmessage("Writing log file")
d = os.path.dirname(DGIDatabase)
out = open(d + "/log.txt","w")

out.write("#Date and Time\n")
out.write(time.asctime()+ "\n")
out.write("#Geodatabase\n")
out.write(DGIDatabase + "\n")
out.write("#Irrigation Wells Shapefile\n")
out.write(irr_wells + "\n")

if irr_wells != "#":
    if gp.Exists("wells"):
        gp.delete("wells")
    gp.addmessage("Loading Wells shapefile")
    gp.CopyFeatures(irr_wells, DGIDatabase + os.sep + "wells")
    out.write('gp.CopyFeatures(irr_wells, DGIDatabase + os.sep + "wells")' + "\n")

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("info_wells"):
        gp.delete("info_wells")
    if gp.Exists("no_info_wells"):
        gp.delete("no_info_wells")
    if gp.Exists("Est_wells"):
        gp.delete("Est_wells")
    gp.CreateTable(DGIDatabase, "Est_wells")
    out.write('gp.CreateTable(DGIDatabase, "Est_wells")' + "\n")
    gp.AddField("Est_wells","WELL_ID","text", "20")
    gp.AddField("Est_wells","WELL_TYPE","text", "20")
    gp.AddField("Est_wells","EST_ID","text", "20")
    gp.AddField("Est_wells","EST_TYPE","text", "20")
 
    gp.MakeFeatureLayer("wells", "wells_lyr")
    gp.SelectLayerByAttribute("wells_lyr", 'new_selection', "BPERF > 0")
    gp.SelectLayerByAttribute("wells_lyr", 'add_to_selection', "DEPTH > 0")
    gp.SelectLayerByAttribute("wells_lyr", 'add_to_selection', "AQUIFER1 <> ''")
    gp.CopyFeatures("wells_lyr", "info_wells")
    out.write('gp.CopyFeatures("wells_lyr", "info_wells")' + "\n")
    gp.SelectLayerByAttribute("wells_lyr", 'switch_selection', "")
    gp.CopyFeatures("wells_lyr", "no_info_wells")
    out.write('gp.CopyFeatures("wells_lyr", "no_info_wells")' + "\n")

    #buffer five miles
    if gp.Exists("no_info_buffer"):
        gp.delete("no_info_buffer")
    gp.buffer("no_info_wells", "no_info_buffer", 8047)
    out.write('gp.buffer("no_info_wells", "no_info_buffer", 8047)' + "\n")
    if gp.Exists("wells_intersect"):
        gp.delete("wells_intersect")
    gp.Intersect_analysis("info_wells;no_info_buffer", "wells_intersect")
    out.write('gp.Intersect_analysis("info_wells;no_info_buffer", "wells_intersect")' + "\n")

    tabCur = gp.InsertCursor("Est_wells")
    
    wRows = gp.SearchCursor("no_info_wells")
    wRow = wRows.next()
    while wRow:
        pRows = gp.SearchCursor("wells_intersect", "WELL_ID_1 = '" + wRow.GetValue("WELL_ID") + "'")
        pRow = pRows.next()
        dist = 999999
        while pRow:
            wp = wRow.Shape.GetPart(0)
            pp = pRow.Shape.GetPart(0)
            dx = wp.x - pp.x
            dy = wp.y - pp.y
            d = math.sqrt(dx*dx + dy*dy)
            if (d < dist):
                id = pRow.GetValue("WELL_ID")
                idtyp = pRow.GetValue("ID_TYPE")
                dist = d
            pRow = pRows.next()

        if (dist < 99999):            
            tRow = tabCur.NewRow()
            tRow.SetValue("WELL_ID",wRow.GetValue("WELL_ID"))
            tRow.SetValue("WELL_TYPE",wRow.GetValue("WELL_ID"))
            tRow.SetValue("EST_ID",id)
            tRow.SetValue("EST_TYPE",idtyp)
            tabCur.InsertRow(tRow)

        wRow = wRows.next()
    
    #del wRow,wRows,tRow,tabCur

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("info_wells"):
        gp.delete("info_wells")
    if gp.Exists("no_info_wells"):
        gp.delete("no_info_wells")
    if gp.Exists("no_info_buffer"):
        gp.delete("no_info_buffer")
    if gp.Exists("wells_intersect"):
        gp.delete("wells_intersect")
    
                
#Perform Model Intersection.
if gp.exists("model"):
    gp.addmessage("Intersecting Input Shapefiles with Model")
   

    if gp.exists("wells"):
        gp.addmessage("Intersecting wells with GW model cells")
        if gp.Exists("wells_cells"):
            gp.delete("wells_cells")
        gp.Intersect_analysis("model ; wells ", "wells_cells")
        out.write('gp.Intersect_analysis("model ; wells ", "wells_cells")' + "\n")
        gp.addfield("wells_cells","DUPLICATE","LONG")
        gp.CalculateField("wells_cells","DUPLICATE",0)


gp.compact(DGIDatabase)

out.close()