# ---------------------------------------------------------------------------
# StateDGI_Well_Parcel_1_20t.py
#
#Intersect wells shapefile with Parcels shapefile and output GW only and
#sprinkler parcels and wells. 
# ---------------------------------------------------------------------------
#
#
# Import system modules
import sys, string, os, win32com.client
import math, time

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Declare script arguments...
irrigWells = sys.argv[1]
wellId = sys.argv[2]
irrigParcels = sys.argv[3]
out_file = sys.argv[4]
append_file = sys.argv[6]

search = 402.5

#Assign work environment
wksp = os.path.dirname(out_file)
gp.CreatePersonalGDB(wksp, "well_parcel.mdb")
gp.Workspace = (wksp + os.sep + "well_parcel.mdb")
#gp.addmessage(wksp + os.sep + "well_parcel.mdb")

DGIDatabase = wksp + os.sep + "well_parcel.mdb"

#Check that model parameters have been selected.
#Delete existing files from database and over-write with new
#model run versions.
if gp.Exists("wells"):
    gp.delete("wells")
gp.addmessage("loading wells shapefile")
gp.FeatureClassToFeatureClass(irrigWells, DGIDatabase,"wells")

if(wellId != "WELL_ID"):
    gp.AddField("wells","WELL_ID","TEXT",20)
    gp.CalculateField("wells","WELL_ID","["+wellId+"]")

gp.MakeFeatureLayer("wells", "wells_lyr")
#gp.SelectLayerByAttribute("wells_lyr", 'new_selection', "USE1 = '1' OR USE2 = '1' OR USE3 = '1'")

gp.CopyFeatures("wells_lyr", "irr_wells")
gp.delete("wells_lyr")
gp.MakeFeatureLayer("irr_wells", "irr_wells_lyr")
gp.SelectLayerByAttribute("irr_wells_lyr", 'new_selection','1 = 1')

if gp.Exists("parcels"):
    gp.delete("parcels")
gp.addmessage("loading Irrigated Parcels shapefile")
gp.FeatureClassToFeatureClass(irrigParcels, DGIDatabase,"parcels")
gp.MakeFeatureLayer("parcels", "parcels_lyr")
gp.SelectLayerByAttribute("parcels_lyr", 'new_selection', "IRRIG_TYPE = 'SPRINKLER'")
gp.SelectLayerByAttribute("parcels_lyr", 'add_to_selection', "SW_COV1 = 0")
if gp.Exists("gw_parcels"):
    gp.delete("gw_parcels")
gp.CopyFeatures("parcels_lyr", "gw_parcels")
gp.delete("parcels_lyr")
gp.MakeFeatureLayer("gw_parcels", "gw_parcels_lyr")
gp.SelectLayerByAttribute("gw_parcels_lyr", 'new_selection','1 = 1')


#Check that model parameters have been selected.
#Delete existing intersection files.
#Intersect selected layers with model output.
#export to geodatabase with assigned names.
#Names must match Access query names
if gp.Exists("wells_parcels"):
    gp.delete("wells_parcels")
gp.addmessage("Intersecting Wells and GW Parcels")
gp.Intersect_analysis("irr_wells ; gw_parcels ", "wells_parcels")
gp.addmessage("Intersection Complete")


#Create Well ID Table and Copy Records from intersection into table.
if gp.Exists("well_parcel"):
    gp.delete("well_parcel")

gp.addmessage("Creating Well Parcel Table")    
gp.CopyRows("wells_parcels", "well_parcel")


desc = gp.Describe("well_parcel")
fields = desc.Fields
field = fields.Next()
field = fields.Next()  #skip OBJECTID

while field:
    if (field.Name != "WELL_ID" and field.Name != "PARCEL_ID"):
        gp.DeleteField("well_parcel",field.Name)
    field = fields.next()

gp.AddField("well_parcel","CLASS","short")
gp.AddField("well_parcel","DISTANCE","float")
gp.CalculateField("well_parcel","CLASS",1)
gp.CalculateField("well_parcel","DISTANCE",0)


# Need to remove matched wells and gw parcels here.

gp.AddJoin("irr_wells_lyr","WELL_ID","well_parcel","WELL_ID")
gp.SelectLayerByAttribute("irr_wells_lyr", 'remove_from_selection',"well_parcel.CLASS = 1")
gp.RemoveJoin("irr_wells_lyr","well_parcel")


#find type 2 matches
li = [101,200,300,search]
gp.addmessage("Searching for type 2 matches")

for s in li:

    if gp.Exists("wells_temp"):
        gp.delete("wells_temp")
    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("buff_wells_intersect"):
        gp.delete("buff_wells_intersect")

    gp.addmessage("Buffer Distance " + str(s))
    #Create a temp shapefile of remaining wells to buffer (1/8 mile)
    # and intersect with the GW irrigated parcels.
    #Create a new shapefile from intersection and copy to table.   

    #gp.addmessage("Buffering Wells Shapefile")
    gp.copyFeatures("irr_wells_lyr", "wells_temp")
    gp.buffer("wells_temp", "wells_buffer", s)
    #gp.addmessage("Intersecting buffered wells and irrigated acres")
    gp.Intersect_analysis("gw_parcels;wells_buffer", "buff_wells_intersect")
    #gp.addmessage("Intersection Complete")
    gp.MakeFeatureLayer("wells_temp", "wells_lyr")
    gp.AddJoin("wells_lyr","WELL_ID","buff_wells_intersect","WELL_ID")
    gp.SelectLayerByAttribute("wells_lyr", 'new_selection',"buff_wells_intersect.PARCEL_ID > 1")
    gp.RemoveJoin("wells_lyr","buff_wells_intersect")
    gp.copyFeatures("wells_lyr", "wells_type2")
        
    #Repeat selection of wells not in masterList from above
    wRows = gp.SearchCursor("wells_type2")
    wRow = wRows.Next()
    
    tabCur = gp.InsertCursor("well_parcel")

    while wRow:
        pRows = gp.SearchCursor("buff_wells_intersect", "WELL_ID = '" + wRow.GetValue("WELL_ID") + "'")
        pRow = pRows.next()
        dist = s
        while pRow:
            d = s
            if(pRow.Shape_Area > 10):
                wp = wRow.Shape.GetPart(0)
                pp = string.split(pRow.Shape.Centroid," ")
                pp_x = string.atof(pp[0])
                pp_y = string.atof(pp[1])
                dx = wp.x - pp_x
                dy = wp.y - pp_y
                d = math.sqrt(dx*dx + dy*dy)
            if (d < dist):
                p = pRow.GetValue("PARCEL_ID")
                dist = d
            pRow = pRows.next()
        if (dist < s):
            tRow = tabCur.NewRow()
            tRow.SetValue("Well_ID",wRow.GetValue("WELL_ID"))
            tRow.SetValue("Parcel_ID",p)
            tRow.SetValue("CLASS",2)
            tRow.SetValue("DISTANCE",dist)
            tabCur.InsertRow(tRow)
            # Remove Well ID from Irrigation wells Selected
            gp.SelectLayerByAttribute("irr_wells_lyr", 'remove_from_selection',"Well_ID = '" + wRow.GetValue("Well_ID") + "'")
    
        wRow = wRows.next()
    

    del wRows,wRow,tabCur

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("wells_type2"):
        gp.delete("wells_type2")
    if gp.Exists("wells_temp"):
        gp.delete("wells_temp")
    if gp.Exists("buff_wells_intersect"):
        gp.delete("buff_wells_intersect")
        



# Match remaining wells with all Parcels

if gp.Exists("wells_parcels"):
    gp.delete("wells_parcels")
if gp.Exists("wells_temp"):
    gp.delete("wells_temp")
gp.copyFeatures("irr_wells_lyr", "wells_temp")
gp.addmessage("Intersecting Remaining wells and all Parcels")
gp.Intersect_analysis("wells_temp ; parcels ", "wells_parcels")
gp.addmessage("Intersection Complete")

featCur = gp.SearchCursor("wells_parcels")
tabCur = gp.InsertCursor("well_parcel")
fRow = featCur.next()

while fRow:
    tRow = tabCur.NewRow()
    tRow.SetValue("Well_ID",fRow.GetValue("WELL_ID"))
    tRow.SetValue("Parcel_ID",fRow.GetValue("Parcel_ID"))
    tRow.SetValue("CLASS",1)
    tRow.SetValue("Distance",0)
    tabCur.InsertRow(tRow)
    # Remove Well ID from Irrigation wells Selected
    gp.SelectLayerByAttribute("irr_wells_lyr", 'remove_from_selection',"Well_ID = '" + fRow.GetValue("Well_ID") + "'")
    fRow = featCur.next()

del featCur,fRow,tabCur

if gp.Exists("wells_temp"):
    gp.delete("wells_temp")
if gp.Exists("wells_layer"):
    gp.delete("wells_layer")
if gp.Exists("buff_wells_intersect"):  
    gp.delete("buff_wells_intersect")

#Create a temp shapefile of remaining wells to buffer (1/4 mile)
    # and intersect with all irrigated parcels.
#Create a new shapefile from intersection and import to database.   


#find type 2 matches
li = [100,200,300,search]
gp.addmessage("Searching for type 2 matches")

for s in li:

    if gp.Exists("wells_temp"):
        gp.delete("wells_temp")
    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("buff_wells_intersect"):
        gp.delete("buff_wells_intersect")

    gp.addmessage("Buffer Distance " + str(s))
    #Create a temp shapefile of remaining wells to buffer (1/8 mile)
    # and intersect with the GW irrigated parcels.
    #Create a new shapefile from intersection and copy to table.   

    #gp.addmessage("Buffering Wells Shapefile")
    gp.copyFeatures("irr_wells_lyr", "wells_temp")
    gp.buffer("wells_temp", "wells_buffer", s)
    #gp.addmessage("Intersecting buffered wells and irrigated acres")
    gp.Intersect_analysis("parcels;wells_buffer", "buff_wells_intersect")
    #gp.addmessage("Intersection Complete")
    gp.MakeFeatureLayer("wells_temp", "wells_lyr")
    gp.AddJoin("wells_lyr","WELL_ID","buff_wells_intersect","WELL_ID")
    gp.SelectLayerByAttribute("wells_lyr", 'new_selection',"buff_wells_intersect.PARCEL_ID > 1")
    gp.RemoveJoin("wells_lyr","buff_wells_intersect")
    gp.copyFeatures("wells_lyr", "wells_type2")
    

    
    #Repeat selection of wells not in masterList from above
    wRows = gp.SearchCursor("wells_type2")
    wRow = wRows.next()
    
    tabCur = gp.InsertCursor("well_parcel")
    
    while wRow:
        #gp.addmessage(wRow.GetValue("WELL_ID"))
        pRows = gp.SearchCursor("buff_wells_intersect", "WELL_ID = '" + wRow.GetValue("WELL_ID") + "'")
        pRow = pRows.next()
        dist = s
        while pRow:
            d = s
            if(pRow.Shape_Area > 10):
                wp = wRow.Shape.GetPart(0)
                pp = string.split(pRow.Shape.Centroid," ")
                pp_x = string.atof(pp[0])
                pp_y = string.atof(pp[1])
                dx = wp.x - pp_x
                dy = wp.y - pp_y
                d = math.sqrt(dx*dx + dy*dy)
            if (d < dist):
                p = pRow.GetValue("PARCEL_ID")
                dist = d
            pRow = pRows.next()
        if (dist < s):
            tRow = tabCur.NewRow()
            tRow.SetValue("Well_ID",wRow.GetValue("WELL_ID"))
            tRow.SetValue("Parcel_ID",p)
            tRow.SetValue("CLASS",2)
            tRow.SetValue("Distance",dist)
            tabCur.InsertRow(tRow)
            # Remove Well ID from Irrigation wells Selected
            gp.SelectLayerByAttribute("irr_wells_lyr", 'remove_from_selection',"Well_ID = '" + wRow.GetValue("Well_ID") + "'")
    
        wRow = wRows.next()
    
    del wRows,wRow,tabCur

    if gp.Exists("wells_lyr"):
        gp.delete("wells_lyr")
    if gp.Exists("wells_type2"):
        gp.delete("wells_type2")
    if gp.Exists("wells_temp"):
        gp.delete("wells_temp")
    if gp.Exists("buff_wells_intersect"):
        gp.delete("buff_wells_intersect")
        


#Type 4 Matches

li = [100,200,500,1000,2000,4000,8047]
gp.addmessage("Searching for type 4 matches")

for s in li:
    gp.addmessage("Buffer Distance " + str(s))
    gp.AddJoin("gw_parcels_lyr","PARCEL_ID","well_parcel","PARCEL_ID")
    gp.SelectLayerByAttribute("gw_parcels_lyr", 'remove_from_selection',"well_parcel.PARCEL_ID > 1")
    gp.RemoveJoin("gw_parcels_lyr","well_parcel")
    gp.copyFeatures("gw_parcels_lyr", "gw_temp")
    gp.buffer("gw_temp", "gw_temp_buffer", s)
    gp.Intersect_analysis("irr_wells ; gw_temp_buffer", "buff_wells_intersect")
    gp.MakeFeatureLayer("gw_temp", "gw_temp_lyr")
    gp.AddJoin("gw_temp_lyr","PARCEL_ID","buff_wells_intersect","PARCEL_ID")
    gp.SelectLayerByAttribute("gw_temp_lyr", 'new_selection',"buff_wells_intersect.PARCEL_ID > 1")
    gp.RemoveJoin("gw_temp_lyr","buff_wells_intersect")
    gp.copyFeatures("gw_temp_lyr", "gw_type4")

    #Repeat selection of wells not in masterList from above
    wRows = gp.SearchCursor("gw_type4")
    wRow = wRows.next()

    tabCur = gp.InsertCursor("well_parcel")

    while wRow:
        pRows = gp.SearchCursor("buff_wells_intersect", "PARCEL_ID = " + str(wRow.GetValue("PARCEL_ID")))
        pRow = pRows.next()
        dist = s
        while pRow:
            d = s
            if(wRow.Shape_Area > 10 ):
                wp = string.split(wRow.Shape.Centroid," ")
                pp = pRow.Shape.GetPart(0)
                pp_x = pp.x
                pp_y = pp.y
                wp_x = string.atof(wp[0])
                wp_y = string.atof(wp[1])
                dx = wp_x - pp_x
                dy = wp_y - pp_y
                d = math.sqrt(dx*dx + dy*dy)
            if (d < dist):
                w = pRow.GetValue("WELL_ID")
                dist = d
            pRow = pRows.next()
        if (dist < s):
            tRow = tabCur.NewRow()
            tRow.SetValue("WELL_ID",w)
            tRow.SetValue("PARCEL_ID",wRow.GetValue("PARCEL_ID"))
            tRow.SetValue("CLASS",4)
            tRow.SetValue("DISTANCE",dist)
            tabCur.InsertRow(tRow)
        wRow = wRows.next()
       
    del wRows,wRow,tabCur

    if gp.Exists("wells_buffer"):
        gp.delete("wells_buffer")
    if gp.Exists("gw_temp"):
        gp.delete("gw_temp")
    if gp.Exists("gw_temp_buffer"):
        gp.delete("gw_temp_buffer")
    if gp.Exists("gw_type4"):
        gp.delete("gw_type4")
    if gp.Exists("gw_temp_lyr"):
        gp.delete("gw_temp_lyr")
    if gp.Exists("buff_wells_intersect"):
        gp.delete("buff_wells_intersect")


# Clean up
if gp.Exists("wells_temp"):
    gp.delete("wells_temp")
if gp.Exists("wells_layer"):
    gp.delete("wells_layer")
if gp.Exists("wells_buffer"):
    gp.delete("wells_buffer")
if gp.Exists("wells_parcels"):
    gp.delete("wells_parcels")
if gp.Exists("irr_wells"):
    gp.delete("irr_wells")
if gp.Exists("buff_wells_intersect"):  
    gp.delete("buff_wells_intersect")
if gp.Exists("gw_parcels"):
    gp.delete("gw_parcels")
if gp.Exists("gw_temp_buffer"):
    gp.delete("gw_temp_buffer")


#Add well matches to parcels table    

wRows = gp.UpdateCursor("parcels")
wRow = wRows.Next()
while wRow:
    pRows = gp.SearchCursor("well_parcel", "PARCEL_ID = " + str(wRow.GetValue("PARCEL_ID")))
    pRow = pRows.Next()
    i = 1
    while pRow:
        wRow.SetValue("GW_ID" + str(i),pRow.GetValue("WELL_ID"))
        wRow.SetValue("GW_CLASS" + str(i),pRow.GetValue("CLASS"))
        wRows.UpdateRow(wRow)
        i = i + 1
        pRow = pRows.Next()

    del pRow,pRows
    wRow = wRows.Next()

del wRow,wRows

gp.addmessage("Writing output file")
out = open(out_file,"w")

out.write("#Date and Time |" + time.asctime()+ "\n")
out.write("#Well Theme |" + irrigWells + "\n")
out.write("#Irrigated Area Theme |" + irrigParcels + "\n")
    
pRows = gp.SearchCursor("well_parcel")
pRow = pRows.Next()

    
while pRow:
    p = pRow.GetValue("PARCEL_ID")
    w = pRow.GetValue("WELL_ID")
    c = pRow.GetValue("CLASS")
    d = pRow.GetValue("DISTANCE")
    a_str = "%.0f" %(d)
    out.write(str(p) + "," + w + "," + str(c) + "," + a_str + "\n")

    pRow = pRows.Next()

del pRow,pRows

out.close()

if append_file != "#":
    gp.CopyFeatures("parcels",append_file)




gp.Compact(DGIDatabase)
