# ---------------------------------------------------------------------------

# Import system modules
import sys, string, math, time, os
#9.1
import win32com.client
#9.2
#import arcgisscripting

# Create the Geoprocessor object
#9.1
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
#9.2
#gp = arcgisscripting.create()

# Check out any necessary licenses

# Script arguments...
irr_parcel = sys.argv[1]
irr_parcel_id = "PARCEL_ID"
agg_areas = sys.argv[2]
agg_id = sys.argv[3]
out_file = sys.argv[4]
append_results = sys.argv[5]
append_file = sys.argv[6]

if (out_file.find(" ") > -1):
    gp.addmessage("Output file has spaces in full file name.  Output file cannot be created")
    raise
try:
    gp.addmessage("Writing log file")
    d = os.path.dirname(out_file)
    out_log = open(d + "/log.txt","w")

    out_log.write("#Date and Time\n")
    out_log.write(time.asctime()+ "\n")
    out_log.write("#Irrigated Parcels Shapefile\n")
    out_log.write(irr_parcel + "\n")
    out_log.write("#Aggregate Areas Shapefile\n")
    out_log.write(agg_areas + "\n")
    out_log.write("#Aggregate Areas ID\n")
    out_log.write(agg_id + "\n")
    out_log.write("#Output File\n")
    out_log.write(out_file + "\n")
    out_log.write("#Output Shapefile\n")
    out_log.write(append_file + "\n")
    
    #gp.addmessage(os.path.dirname(irr_parcel))
    #gp.addmessage(os.path.dirname(agg_areas))
    wksp = os.path.dirname(out_file)
    gp.CreatePersonalGDB(wksp, "agg_str.mdb")
    gp.Workspace = (wksp + os.sep + "agg_str.mdb")
    gp.addmessage(wksp + os.sep + "agg_str.mdb")
    

    gp.addmessage("Intersecting Input Coverage with Aggregate Area")
    if gp.Exists("agg_intersect"):
        gp.delete("agg_intersect")
    gp.Intersect_analysis(irr_parcel + ";" + agg_areas, "agg_intersect")
    out_log.write("gp.Intersect_analysis(irr_parcel + ';' + agg_areas, 'agg_intersect')\n")


    #gp.addmessage("Creating irr_str table")
    if gp.Exists("irr_str"):
        gp.delete("irr_str")
    gp.CreateTable(wksp + os.sep + "agg_str.mdb", "irr_str")
    out_log.write('gp.CreateTable(wksp + os.sep + "agg_str.mdb", "irr_str")\n')
    gp.AddField("irr_str","comp_id","text",20)
    out_log.write('gp.AddField("irr_str","comp_id","text",20)\n')
    gp.AddField("irr_str","SW_WDID","text", "20")
    out_log.write('gp.AddField("irr_str","SW_WDID","text", "20")\n')
    gp.AddField("irr_str","pro_area","double")
    wCur = gp.InsertCursor("irr_str")
    pRows = gp.SearchCursor("agg_intersect")
    pRow = pRows.Next()
    i = 0
    #added GW_ID1 to exclude parcels without any water source && 
    while pRow:
        if pRow.GetValue("SW_COV1") == 0 and pRow.GetValue("GW_ID1") != 0:
            wRow = wCur.NewRow()
            wRow.SW_WDID = str(int(pRow.GetValue("PARCEL_ID")))
            wRow.SetValue("pro_area",pRow.GetValue("Shape_Area"))
            id = str(int(pRow.GetValue("PARCEL_ID"))) + "|" + str(pRow.GetValue(agg_id))
            wRow.SetValue("comp_id",id)
            wCur.InsertRow(wRow)
            i = 1
        pRow = pRows.Next()        
            
    if (i == 0):
        gp.addmessage("No GW only Parcels found!")
        raise "exit"

    del wCur,wRow,pRows,pRow

    # Get Total areas
    gp.Statistics("irr_str", "tot_stats","pro_area sum","SW_WDID")
    out_log.write('gp.Statistics("irr_str", "tot_stats","pro_area sum","SW_WDID")\n')
    gp.Statistics("irr_str", "agg_stats","pro_area sum","comp_id")
    out_log.write('gp.Statistics("irr_str", "agg_stats","pro_area sum","comp_id")\n')

    gp.AddField("agg_stats","SW_WDID","text",20)
    gp.AddField("agg_stats","area","double")
    gp.AddField("agg_stats","total","double")
    gp.AddField("agg_stats","max","double")
    gp.CalculateField("agg_stats","area","[sum_pro_area]")
    out_log.write('gp.CalculateField("agg_stats","area","[sum_pro_area]")\n')
    gp.DeleteField("agg_stats","sum_pro_area")

    pRows = gp.UpdateCursor("agg_stats")
    pRow = pRows.Next()
    while pRow:
        id = pRow.GetValue("comp_id").split("|")
        pRow.SW_WDID = id[0]
        pRows.UpdateRow(pRow)
        pRow = pRows.Next()
    del pRow,pRows

    gp.Statistics("agg_stats", "agg_stats_max","area max","SW_WDID")
    out_log.write('gp.Statistics("agg_stats", "agg_stats_max","area max","SW_WDID")\n')

    gp.MakeTableView("agg_stats" , "tmp")
    gp.AddJoin("tmp","SW_WDID","tot_stats","SW_WDID")
    gp.CalculateField("tmp","agg_stats.total","[tot_stats.sum_pro_area]")
    out_log.write('gp.CalculateField("tmp","agg_stats.total","[tot_stats.sum_pro_area]")\n')
    gp.RemoveJoin("tmp", "tot_stats")

    gp.AddJoin("tmp","SW_WDID","agg_stats_max","SW_WDID")
    gp.CalculateField("tmp","agg_stats.max","[agg_stats_max.max_area]")
    gp.RemoveJoin("tmp", "agg_stats_max")

    
    gp.addmessage("Writing output file")
    out = open(out_file,"w")

    out.write("#Date and Time |" + time.asctime()+ "\n")
    out.write("#Input Theme |" + irr_parcel + "\n")
    out.write("#Input Theme ID |" + agg_id + "\n")
    out.write("#Aggregate Area Theme |" + agg_areas + "\n")
    out.write("#Aggregate Area ID |" + irr_parcel_id + "|total_acres\n")
    
    pRows = gp.SearchCursor("agg_stats")
    pRow = pRows.Next()

    
    while pRow:
        c = pRow.GetValue("comp_id")
        a = pRow.GetValue("area")
        t = pRow.GetValue("total")
        m = pRow.GetValue("max")
        if a == m:
            id = c.split("|")
            a_str = "%.2f" %(t/4046.95)
            out.write(id[1] + "," + id[0] + "," + a_str + "\n")
        
        pRow = pRows.Next()

    del pRow,pRows

    out.close()
    out_log.write('out.close()\n')
    
    if append_results:
        f = gp.ListFields(irr_parcel)
        ff = f.Next()
        while ff:
            if ff.Name == irr_parcel_id:
                #gp.addmessage(ff.Type)
                irr_parcel_type = ff.Type
                irr_parcel_len = ff.Length
            ff = f.Next()

        f = gp.ListFields(agg_areas)
        ff = f.Next()
        while ff:
            if ff.Name == agg_id:
                #gp.addmessage(ff.Type)
                agg_type = ff.Type
                agg_len = ff.Length
            ff = f.Next()

    
        #Yet another major bug - Thanks ESRI!
        if (irr_parcel_type == 'String'):
            irr_parcel_type = 'TEXT'
        if (agg_type == 'String'):
            agg_type = 'TEXT'
        if (irr_parcel_type == 'Integer'):
            irr_parcel_type = 'LONG'
        if (agg_type == 'Integer'):
            agg_type = 'LONG'

        gp.CopyFeatures(irr_parcel,"append_file")
        time.sleep(5)
        out_log.write('gp.CopyFeatures(irr_parcel,append_file)\n')
        gp.AddField("append_file",agg_id,agg_type,agg_len)


        if gp.Exists("summary"):
            gp.delete("summary")
        gp.CreateTable(wksp + os.sep + "agg_str.mdb", "summary")
        gp.AddField("summary","irr_parcel_id",irr_parcel_type,irr_parcel_len)
        gp.AddField("summary","agg_id",agg_type,agg_len)
        wCur = gp.InsertCursor("summary")
        pRows = gp.SearchCursor("agg_stats")
        pRow = pRows.Next()
        while pRow:
            c = pRow.GetValue("comp_id")
            a = pRow.GetValue("area")
            t = pRow.GetValue("total")
            m = pRow.GetValue("max")
            if a == m:
                id = c.split("|")
                wRow = wCur.NewRow()
                wRow.irr_parcel_id = pRow.GetValue("SW_WDID")
                wRow.agg_id = id[1]
                wCur.InsertRow(wRow)
            pRow = pRows.Next()        
            
        del wCur,wRow,pRows,pRow

        if gp.Exists("tmp_lyr"):
            gp.delete("tmp_lyr")
        gp.MakeFeatureLayer("append_file", "tmp_lyr")
        gp.AddJoin("tmp_lyr",irr_parcel_id,"summary","irr_parcel_id")
        gp.SelectLayerByAttribute("tmp_lyr", "NEW_SELECTION", "[summary.agg_id] IS NOT NULL")
        gp.CalculateField("tmp_lyr",agg_id,"[summary.agg_id]")
        gp.RemoveJoin("tmp_lyr", "summary")
        gp.CopyFeatures("append_file",append_file)


    out_log.close()
    
except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
