# ---------------------------------------------------------------------------

# Import system modules
import sys, string, os, math, time, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Check out any necessary licenses
gp.CheckOutExtension("spatial")

# Script arguments...
theme = sys.argv[1]
Zone_field = sys.argv[2]
rast = sys.argv[3]
out_file = sys.argv[4]
append_results = sys.argv[5]
append_file = sys.argv[6]

Ignore_NoData_in_calculations = "true"

try:

    f = gp.ListFields(theme)
    ff = f.Next()
    while ff:
        if ff.Name == Zone_field:
            Zone_type = ff.Type
            Zone_len = ff.Length
        ff = f.Next()
    
    #Yet another major bug - Thanks ESRI!
    if (Zone_type == 'String'):
        Zone_type = 'TEXT'
    if (Zone_type == 'Integer'):
        Zone_type = 'LONG'
    if (Zone_type == 'SmallInteger'):
        Zone_type = 'LONG'

    #gp.Workspace = os.path.dirname(out_file)
    wksp = os.path.dirname(out_file)
    gp.CreatePersonalGDB(wksp, "avg_pcp.mdb")
    gp.Workspace = (wksp + os.sep + "avg_pcp.mdb")
    gp.addmessage(wksp + os.sep + "avg_pcp.mdb")
        
    gp.addmessage("Processing grid..")
    # Local variables...
    ResultsTable = "ZonalStats"
    # Process: Zonal Statistics as Table...
    gp.ZonalStatisticsAsTable_sa(theme, Zone_field, rast, ResultsTable, Ignore_NoData_in_calculations)
    
    gp.addmessage("Writing output file")
    out = open(out_file,"w")

    out.write("#Date and Time |" + time.asctime()+ "\n")
    out.write("#Basin Theme |" + theme + "\n")
    out.write("#Basin Theme ID |" + Zone_field + "\n")
    out.write("#Precipitation Grid |" + rast + "\n")
    out.write("#Output File |" + out_file + "\n")
    out.write("#Basin ID |Average Precip|total_acres\n")
    
    pRows = gp.SearchCursor(ResultsTable)
    pRow = pRows.Next()

    if (Zone_type == 'TEXT'):
        fld_name = Zone_field
    else:
        fld_name = "VALUE_"
    
    while pRow:
        c = pRow.GetValue(fld_name)
        a = pRow.GetValue("AREA")
        t = pRow.GetValue("MEAN")
        a_str = "%.2f" %(a/4046.95)
        out.write(str(c) + "," + str(t) + "," + a_str + "\n")
        
        pRow = pRows.Next()

    del pRow,pRows

    out.close()


    if append_results:

        gp.CopyFeatures(theme,"append_file")
        gp.AddField("append_file","avgprecip","double",20,2)

        if gp.Exists("tmp_lyr"):
            gp.delete("tmp_lyr")
        gp.MakeFeatureLayer("append_file", "tmp_lyr")
        gp.AddJoin("tmp_lyr",Zone_field,ResultsTable,fld_name)
        #gp.SelectLayerByAttribute("tmp_lyr", "NEW_SELECTION", "[ZonalStats.MEAN] IS NOT NULL")
        gp.CalculateField("tmp_lyr","avgprecip","[ZonalStats.MEAN]")
        gp.RemoveJoin("tmp_lyr", ResultsTable)

        gp.CopyFeatures("append_file",append_file)



    
    #gp.Delete(ResultsTable)
        
except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
