# ---------------------------------------------------------------------------

# Import system modules
import sys, string, os, math, time, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Check out any necessary licenses
gp.CheckOutExtension("spatial")

# Script arguments...
theme = sys.argv[1]
Zone_field = sys.argv[2]
rast = sys.argv[3]
out_file = sys.argv[4]
append_results = sys.argv[5]
append_file = sys.argv[6]

Ignore_NoData_in_calculations = "true"

try:

    f = gp.ListFields(theme)
    ff = f.Next()
    while ff:
        if ff.Name == Zone_field:
            Zone_type = ff.Type
            Zone_len = ff.Length
        ff = f.Next()
    del f,ff
    
    #Yet another major bug - Thanks ESRI!
    if (Zone_type == 'String'):
        Zone_type = 'TEXT'
    if (Zone_type == 'Integer'):
        Zone_type = 'LONG'
    if (Zone_type == 'SmallInteger'):
        Zone_type = 'LONG'

    #gp.Workspace = os.path.dirname(out_file)
    wksp = os.path.dirname(out_file)
    gp.CreatePersonalGDB(wksp, "avg_pcp.mdb")
    gp.Workspace = (wksp + os.sep + "avg_pcp.mdb")
    gp.addmessage(wksp + os.sep + "avg_pcp.mdb")

    gp.CopyFeatures(theme,"append_file")
    gp.AddField("append_file","avgprecip","double",20,2)
    gp.MakeFeatureLayer("append_file","append_file_lyr")
    gp.Statistics("append_file","uniq_ids","Shape_Area sum",Zone_field)

    if (Zone_type == 'TEXT'):
        fld_name = Zone_field
    else:
        fld_name = "VALUE_"

    uRows = gp.SearchCursor("uniq_ids")
    uRow = uRows.Next()
    while uRow:
        if (Zone_type == 'TEXT'):
            gp.SelectLayerByAttribute("append_file_lyr", 'new_selection',Zone_field + " = '" + str(uRow.GetValue(Zone_field)) + "'")
        else:
            gp.SelectLayerByAttribute("append_file_lyr", "new_selection",Zone_field + " = " + str(uRow.GetValue(Zone_field)))
        ResultsTable = str(uRow.GetValue(Zone_field))
        gp.addmessage("Processing grid for attribute value " + ResultsTable)
        gp.ZonalStatisticsAsTable("append_file_lyr", Zone_field, rast, ResultsTable, Ignore_NoData_in_calculations)
        uRow = uRows.Next()
    del uRow,uRows

    uRows = gp.SearchCursor("uniq_ids")
    uRow = uRows.Next()
    while uRow:
        if (Zone_type == 'TEXT'):
            gp.SelectLayerByAttribute("append_file_lyr", 'new_selection',Zone_field + " = '" + str(uRow.GetValue(Zone_field)) + "'")
        else:
            gp.SelectLayerByAttribute("append_file_lyr", "new_selection",Zone_field + " = " + str(uRow.GetValue(Zone_field)))
        ResultsTable = str(uRow.GetValue(Zone_field))
        gp.AddJoin("append_file_lyr",Zone_field,ResultsTable,fld_name)
        gp.CalculateField("append_file_lyr","avgprecip","["+ ResultsTable + ".MEAN]")
        gp.RemoveJoin("append_file_lyr", ResultsTable)
        #gp.Delete(ResultsTable)
        uRow = uRows.Next()
    del uRow,uRows

    gp.Statistics("append_file","results","avgprecip first; Shape_Area sum",Zone_field)

    gp.addmessage("Writing output file")
    out = open(out_file,"w")

    out.write("#Date and Time |" + time.asctime()+ "\n")
    out.write("#Basin Theme |" + theme + "\n")
    out.write("#Basin Theme ID |" + Zone_field + "\n")
    out.write("#Precipitation Grid |" + rast + "\n")
    out.write("#Output File |" + out_file + "\n")
    out.write("#Basin ID |Average Precip|total_acres\n")
    
    pRows = gp.SearchCursor("results")
    pRow = pRows.Next()
        
    while pRow:
        c = pRow.GetValue(Zone_field)
        a = pRow.GetValue("sum_Shape_Area")
        t = pRow.GetValue("first_avgprecip")
        a_str = "%.2f" %(a/4046.95)
        out.write(str(c) + "," + str(t) + "," + a_str + "\n")
        
        pRow = pRows.Next()

    del pRow,pRows

    out.close()


    if append_results:
        gp.CopyFeatures("append_file",append_file)

        
except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
