# ---------------------------------------------------------------------------

# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Check out any necessary licenses

# Script arguments...
irr_parcel = sys.argv[1]
parcel_id = sys.argv[2]
soil_areas = sys.argv[3]
awc_id = sys.argv[4]
out_file = sys.argv[5]
append_results = sys.argv[6]
append_file = sys.argv[7]

try:
    wksp = os.path.dirname(out_file)
    gp.CreatePersonalGDB(wksp, "awc_str.mdb")
    gp.Workspace = (wksp + os.sep + "awc_str.mdb")
    gp.addmessage(wksp + os.sep + "awc_str.mdb")

    inDesc = gp.describe(irr_parcel)
    if (inDesc.ShapeType.lower() == 'polygon'):
        a_str = "Shape_Area"
    else:
        a_str = "Shape_Length"
    #gp.addmessage(a_str)
    

    gp.addmessage("Intersecting input coverage with Soils Coverage")
    if gp.Exists("awc_intersect"):
        gp.delete("awc_intersect")
    gp.Intersect_analysis(irr_parcel + ";" + soil_areas, "awc_intersect")
    gp.AddField("awc_intersect","pro_area","double")
    gp.CalculateField("awc_intersect","pro_area","[" + a_str + "] * [" + awc_id + "]")

    # Get Total areas
    gp.Statistics("awc_intersect", "awc_stats",a_str + " sum;pro_area sum",parcel_id)
    
    if out_file != "#":
        gp.addmessage("Writing output file")
        out = open(out_file,"w")

        out.write("#Date and Time |" + time.asctime()+ "\n")
        out.write("#Shapefile |" + irr_parcel + "\n")
        out.write("#Shapefile ID |" + parcel_id + "\n")
        out.write("#Soil AWC Theme |" + soil_areas + "\n")
        out.write("#AWC_ID |" + awc_id + "\n")
        out.write(str(parcel_id) + "," + awc_id + "\n")
    
        pRows = gp.SearchCursor("awc_stats")
        pRow = pRows.Next()
    
        while pRow:
            id = pRow.GetValue(parcel_id)
            a = pRow.GetValue("sum_pro_area")
            t = pRow.GetValue("sum_" + a_str)
            awc_str = "%.4f" %(a/t)
            out.write(str(id) + "," + awc_str + "\n")
            pRow = pRows.Next()
        del pRow,pRows

        out.close()
    
    if append_results:
        if gp.Exists("tmp_lyr"):
            gp.delete("tmp_lyr")
        gp.AddField(irr_parcel,awc_id,"double")
        gp.MakeFeatureLayer(irr_parcel, "tmp_lyr")
        gp.AddJoin("tmp_lyr",parcel_id,"awc_stats",parcel_id)
        gp.SelectLayerByAttribute("tmp_lyr", "NEW_SELECTION", "[awc_stats.sum_pro_area] IS NOT NULL")
        gp.CalculateField("tmp_lyr",awc_id,"[awc_stats.sum_pro_area]/[awc_stats.sum_"+a_str+"]")
        gp.RemoveJoin("tmp_lyr", "awc_stats")
        gp.CopyFeatures(irr_parcel,append_file)
        gp.DeleteField(irr_parcel,awc_id)
        
except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
