# ---------------------------------------------------------------------------

# Import system modules
import sys, string, math, time, os, win32com.client

# Create the Geoprocessor object
gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")

# Check out any necessary licenses

# Script arguments...
irr_parcel = sys.argv[1]
soil_areas = sys.argv[2]
awc_id = sys.argv[3]
out_file = sys.argv[4]
append_results = sys.argv[5]
append_file = sys.argv[6]

try:
    wksp = os.path.dirname(out_file)
    gp.CreatePersonalGDB(wksp, "awc_str.mdb")
    gp.Workspace = (wksp + os.sep + "awc_str.mdb")
    gp.addmessage(wksp + os.sep + "awc_str.mdb")
    

    gp.addmessage("Intersecting Irrigated Parcels with Soils Coverage")
    if gp.Exists("awc_intersect"):
        gp.delete("awc_intersect")
    gp.Intersect_analysis(irr_parcel + ";" + soil_areas, "awc_intersect")


    gp.addmessage("Creating irr_str table")
    if gp.Exists("irr_str"):
        gp.delete("irr_str")
    gp.CreateTable(wksp + os.sep + "awc_str.mdb", "irr_str")
    gp.AddField("irr_str","SW_WDID","text", "20")
    gp.AddField("irr_str","pro_area","double")
    gp.AddField("irr_str","awc_area","double")
    wCur = gp.InsertCursor("irr_str")
    pRows = gp.SearchCursor("awc_intersect")
    pRow = pRows.Next()
    while pRow:
        for i in range(9):
            if pRow.GetValue("SW_COV" + str(i+1)) > 0:
                wRow = wCur.NewRow()
                wRow.SW_WDID = pRow.GetValue("SW_WDID" + str(i+1))
                p_a = pRow.GetValue("SW_COV" + str(i+1))/100*pRow.GetValue("Shape_Area")
                wRow.SetValue("pro_area",p_a)
                wRow.SetValue("awc_area",pRow.GetValue(awc_id) * p_a)
                wCur.InsertRow(wRow)
        pRow = pRows.Next()
            
    del wCur,wRow,pRows,pRow

    # Get Total areas
    gp.Statistics("irr_str", "awc_stats","pro_area sum;awc_area sum","SW_WDID")
    
    gp.addmessage("Writing output file")
    out = open(out_file,"w")

    out.write("#Date and Time |" + time.asctime()+ "\n")
    out.write("#Irrigated Parcel Theme |" + irr_parcel + "\n")
    out.write("#Soil AWC Theme |" + soil_areas + "\n")
    out.write("#AWC ID |" + awc_id + "\n")
    
    pRows = gp.SearchCursor("awc_stats")
    pRow = pRows.Next()

    
    while pRow:
        id = pRow.GetValue("SW_WDID")
        t = pRow.GetValue("sum_pro_area")
        a = pRow.GetValue("sum_awc_area")
        awc_str = "%.4f" %(a/t)
        out.write(id + "," + awc_str + "\n")
        pRow = pRows.Next()

    del pRow,pRows

    out.close()
    
    if append_results:
        f = gp.ListFields(soil_areas)
        ff = f.Next()
        while ff:
            if ff.Name == awc_id:
                awc_type = ff.Type
                awc_len = ff.Length
            ff = f.Next()
    
        #Yet another major bug - Thanks ESRI!
        if (awc_type == 'String'):
            awc_type = 'TEXT'
        if (awc_type == 'Integer'):
            awc_type = 'LONG'

        gp.CopyFeatures(irr_parcel,"append_file")
        gp.AddField("append_file",awc_id,awc_type,awc_len)

        if gp.Exists("tmp_lyr"):
            gp.delete("tmp_lyr")
        gp.MakeFeatureLayer("append_file", "tmp_lyr")
        gp.AddJoin("tmp_lyr","SW_WDID1","awc_stats","SW_WDID")        
        gp.SelectLayerByAttribute("tmp_lyr", "NEW_SELECTION", "[append_file.SW_COV1] > 0")
        gp.CalculateField("tmp_lyr",awc_id,"[awc_stats.sum_awc_area]/[awc_stats.sum_pro_area]")
        gp.RemoveJoin("tmp_lyr", "awc_stats")
        gp.CopyFeatures("append_file",append_file)
    
   

        
except:
    #If an error occurred while running a tool, print the messages
    print gp.GetMessages(2)
